from os import path
import os

__all__ = ['env']

class _Env:
	base_dir = path.dirname(path.dirname(path.abspath(__file__)))
	skel_dir = path.join(base_dir, 'skel')
	code_dir = path.join(base_dir, 'code')
	package_dir = os.getenv('CIPR_PACKAGES', None) or path.expanduser('~/.cipr/packages')
	dir = path.abspath(os.getcwd())
	dist_dir = path.join(dir, '..', 'dist', 'Payload')
	build_dir = path.join(dir, '..', 'build')

env = _Env()


import clik
from os import path
import os
from optparse import make_option as opt
from cipr.commands.cfg import CiprCfg

class _Env(object):
    pass

def _args(opts):
    env = _Env()
    env.base_dir = path.dirname(path.dirname(path.abspath(__file__)))
    env.skel_dir = path.join(env.base_dir, 'skel')
    env.code_dir = path.join(env.base_dir, 'code')
    env.project_directory = opts.project_directory
    env.package_dir = os.getenv('CIPR_PACKAGES', None) or path.join(env.project_directory, '.cipr/packages')    
    env.dist_dir = path.join(env.project_directory, 'dist', 'Payload')
    env.build_dir = path.join(env.project_directory, 'build')

    return dict(
        env = env,
        ciprcfg = CiprCfg(path.join(env.project_directory, '.ciprcfg'))
    )

command = clik.App('cipr',
    version='0.8',
    description='Corona SDK package manager.',
    console_opts=True,
    conf_enabled=False,
    opts= opt('-d', '--project',
        dest='project_directory', default=path.abspath(os.getcwd()),
        help='Project directory'
    ),
    args_callback=_args
)


from os import path
import os
import json

class Package(object):    
    def __init__(self, dir, source):
        self._dir = dir
        self._packagesDir = path.dirname(dir)
        self.info = {}
        self.source = source

        self.name = path.basename(self._dir)

        info_file = path.join(self._dir, 'package.json')
        if path.exists(info_file):
            with open(info_file) as file:
                self.info = json.load(file)

    @property
    def path(self):
        return self._dir

    @property
    def deploy_packages(self):
        return [
            Package(path.join(self._packagesDir, name), source)
                for name, source in self.dependencies.items()
        ]

    @property
    def dependencies(self):
        return self.info.get('dependencies', {})

class CiprCfg(object):
    """
    Manages a project's .ciprcfg file.
    """
    def __init__(self, ciprfile):
        self._filename = ciprfile
        self._data = {}
        if path.exists(self._filename):
            with open(self._filename) as file:
                self._data = json.load(file)

    @property
    def exists(self):
        return path.exists(self._filename)

    def create(self):
        dir = path.dirname(self._filename)
        if not path.exists(dir):
            os.makedirs(dir)

        self._save()

    def _save(self):
        data = json.dumps(self._data, indent=2)
        with open(self._filename, 'w') as file:
            file.write(data)

    @property
    def packages(self):
        """
        Returns a list of packages available to this project
        """
        return self._data.get('packages', [])

    def remove_package(self, name):
        if name in self.packages:
            del self.packages[name]
            self._save()
            
    def add_package(self, package):
        """
        Add a package to this project
        """
        self._data.setdefault('packages', {})
        
        self._data['packages'][package.name] = package.source

        for package in package.deploy_packages:
            self.add_package(package)

        self._save()


import math
import Image
from os import path
from cipr.commands import app

def scale_and_fit(im, requested_size):
    im = im.copy()

    x, y   = [float(v) for v in im.size]
    xr, yr = [float(v) for v in requested_size]

    ratio = min(xr/x, yr/y)

    width = int(math.ceil(x * ratio))
    height = int(math.ceil(y * ratio))

    width, height = int(width), int(height)
    if width != x or height != y:
        im.thumbnail((width, height), resample=Image.ANTIALIAS)

    return im

def scale_and_crop(im, requested_size):
    im = im.copy()

    x, y   = [float(v) for v in im.size]
    xr, yr = [float(v) for v in requested_size]

    ratio = max(xr/x, yr/y)

    width = int(math.ceil(x * ratio))
    height = int(math.ceil(y * ratio))

    width, height = int(width), int(height)
    if width != x or height != y:
        im.thumbnail((width, height), resample=Image.ANTIALIAS)

    x, y   = [float(v) for v in im.size]
    ex, ey = (x-min(x, xr))/2, (y-min(y, yr))/2
    if ex or ey:
        s = (int(math.ceil(ex)), int(math.ceil(ey)), int(math.ceil(x-ex)), int(math.ceil(y-ey)))
        im = im.crop(s)

    return im
    
icon_sizes = {
    # 'iTunesArtwork' : ('icon', 512, 512, scale_and_crop),
    'Icon.png' : ('icon', 57, 57, scale_and_crop),
    'Icon@2x.png' : ('icon', 114, 114, scale_and_crop),
    'Icon-72.png' : ('icon', 72, 72, scale_and_crop),
    'Icon-Small.png' : ('icon', 29, 29, scale_and_crop),
    'Icon-Small@2x.png' : ('icon', 58, 58, scale_and_crop),
    'Icon-Small-50.png' : ('icon', 50, 50, scale_and_crop),
}

@app.command
def makeicons(source):
    """
    Create all the neccessary icons from source image
    """
    im = Image.open(source)
    for name, (_, w, h, func) in icon_sizes.iteritems():
        print('Making icon %s...' % name)
        tn = func(im, (w, h))
        bg = Image.new('RGBA', (w, h), (255, 255, 255))
        x = (w / 2) - (tn.size[0] / 2)
        y = (h / 2) - (tn.size[1] / 2)
        bg.paste(tn, (x, y))

        bg.save(path.join(env.dir, name))            


from os import path
import os
import shutil
from clom import clom, AND
import json
import tempfile
import shutil
from glob import glob
from cipr.commands.cfg import Package, CiprCfg
from cipr.commands import env, util
from cipr.commands import app


@app.command
def init(ciprcfg, env, console):
    """
    Initialize a Corona project directory.
    """
    ciprcfg.create()
    
    templ_dir = path.join(env.skel_dir, 'default')

    console.quiet('Copying files from %s' % templ_dir)
    for src, dst in util.sync_dir_to(templ_dir, env.project_directory, ignore_existing=True):
        console.quiet('  %s -> %s' % (src, dst))

    src = path.join(env.code_dir, 'cipr.dev.lua')
    dst = path.join(env.project_directory, 'cipr.lua')
    console.quiet('  %s -> %s' % (src, dst))

    shutil.copy(src, dst)

@app.command
def update(env):
    """
    Update an existing cipr project to the latest intalled version.
    """
    files = [path.join(env.project_directory, 'cipr.lua')]
    for filename in files:
        if path.exists(filename):
            os.remove(filename)
    app.command.run(['init', env.project_directory])

@app.command(usage='[options] PACKAGE [PACKAGE...]')
def uninstall(args, env, console, ciprcfg):
    """
    Remove a package
    """
    for name in args:
        package_dir = path.join(env.package_dir, name)
        if path.exists(package_dir):
            console.quiet('Removing %s...' % name)
            if path.islink(package_dir):
                os.remove(package_dir)
            else:
                shutil.rmtree(package_dir)

        ciprcfg.remove_package(name)


def _package_info(package):
    version = None
    type = 'dir'
    if package.startswith('git'):
        if '@' in package:
            package, version = package.split('@', 1)

        name = path.splitext(path.basename(package))[0]
        type = 'git'
    else:
        package = path.abspath(package)
        name = path.basename(package)

    return package, name, version, type


@app.command(
    opts=(app.opt('-u', '--upgrade', action='store_true', dest='upgrade', help='Force upgrade of installed package')),
    usage='[options] [PACKAGE] [PACKAGE...]'
)
def install(args, console, env, ciprcfg, opts):
    """
    Install a package from github and make it available for use.
    """
    if len(args) == 0:
        # Is this a cipr project?
        if ciprcfg.exists:
            # Install all the packages for this project
            console.quiet('Installing current project packages...')
            for name, source in ciprcfg.packages.items():
                if opts.upgrade:
                    app.command.run(['install', '--upgrade', source])
                else:
                    app.command.run(['install', source])
        else:
            console.error('No cipr project or package found.')
        return
    else:
        for source in args:
            package, name, version, type = _package_info(source)

            if not path.exists(env.package_dir):
                os.makedirs(env.package_dir)

            package_dir = path.join(env.package_dir, name)

            if path.exists(package_dir):
                if opts.upgrade:
                    app.command.run(['uninstall', name])
                else:
                    console.quiet('Package %s already exists. Use --upgrade to force a re-install.' % name)
                    return

            console.quiet('Installing %s...' % name)


            if type == 'git':        
                tmpdir = tempfile.mkdtemp(prefix='cipr')
                clom.git.clone(package, tmpdir).shell.execute()

                if version:
                    cmd = AND(clom.cd(tmpdir), clom.git.checkout(version))
                    cmd.shell.execute()

                package_json = path.join(tmpdir, 'package.json')
                if path.exists(package_json):
                    # Looks like a cipr package, copy directly
                    shutil.move(tmpdir, package_dir)
                else:
                    # Not a cipr package, sandbox in sub-directory
                    shutil.move(tmpdir, path.join(package_dir, name))

                console.quiet('`%s` installed from git repo to `%s`' % (name, package_dir))

            elif path.exists(package):
                # Local        
                os.symlink(package, package_dir)
            else:
                console.error('Package `%s` type not recognized' % package)
                return
            
            pkg = Package(package_dir, source)
            ciprcfg.add_package(pkg)

            if pkg.dependencies:
                console.quiet('Installing dependancies...')
                for name, require in pkg.dependencies.items():
                    if opts.upgrade:
                        app.command.run(['install', '--upgrade', require])
                    else:
                        app.command.run(['install', require])

@app.command(opts=(app.opt('-l', '--long', action='store_true', dest='long_details', help='List details about package')))
def packages(ciprcfg, env, opts, console):
    """
    List installed packages for this project
    """
    for name, source in ciprcfg.packages.items():
        console.normal('- %s' % name)

        if opts.long_details:
            console.normal('  - directory: %s' % path.join(env.package_dir, name))        
            console.normal('  - source: %s' % source)        
            
    
@app.command
def run(env):
    """
    Run current project in the Corona Simulator
    """
    os.putenv('CIPR_PACKAGES', env.package_dir)
    os.putenv('CIPR_PROJECT', env.project_directory)
    cmd = clom['/Applications/CoronaSDK/Corona Terminal'](env.project_directory)

    try:
        cmd.shell.execute()
    except KeyboardInterrupt:
        pass

@app.command
def build(env, ciprcfg, console):
    """
    Build the current project for distribution
    """
    os.putenv('CIPR_PACKAGES', env.package_dir)
    os.putenv('CIPR_PROJECT', env.project_directory)
        
    if path.exists(env.build_dir):
        shutil.rmtree(env.build_dir)
        
    os.makedirs(env.build_dir)
        
    for src, dst in util.sync_dir_to(env.project_directory, env.build_dir, exclude=['.cipr', '.git']):
        console.quiet('  %s -> %s' % (src, dst))
    
    for package in ciprcfg.packages.keys():
        for src, dst in util.sync_lua_dir_to(path.join(env.package_dir, package), env.build_dir, exclude=['.git'], include=['*.lua']):
            console.quiet('  %s -> %s' % (src, dst))        
    
    src = path.join(env.code_dir, 'cipr.lua')
    dst = path.join(env.build_dir, 'cipr.lua')
    shutil.copy(src, dst)
        
    cmd = AND(clom.cd(env.build_dir), clom['/Applications/CoronaSDK/Corona Terminal'](env.build_dir))

    try:
        cmd.shell.execute()
    except KeyboardInterrupt:
        pass
        
@app.command 
def packageipa(env, console):    
    """
    Package the built app as an ipa for distribution in iOS App Store
    """
    filenames = glob(path.join(env.dist_dir, '*.app'))
    filename, ext = path.splitext(path.basename(filenames[0]))
    ipa_name = filename + '.ipa'
    output_dir = path.dirname(env.dist_dir)
    ipa_path = path.join(output_dir, ipa_name)
    
    if path.exists(ipa_path):
        console.quiet('Removing %s' % ipa_path)
        os.remove(ipa_path)
        
    cmd = AND(clom.cd(output_dir), clom.zip(r=ipa_name).with_args('Payload/%s.app' % filename))
    cmd.shell.execute()


    console.quiet('Packaged %s' % ipa_path)

@app.command 
def expanddotpaths(env, console):        
    """
    Move files with dots in them to sub-directories
    """
    for filepath in os.listdir(path.join(env.dir)):
        filename, ext = path.splitext(filepath)
        if ext == '.lua' and '.' in filename:
            paths, newfilename = filename.rsplit('.', 1)
            newpath = paths.replace('.', '/')
            newfilename = path.join(newpath, newfilename) + ext

            console.quiet('Move %s to %s' % (filepath, newfilename))

            fullpath = path.join(env.project_directory, newpath)
            if not path.exists(fullpath):
                os.makedirs(fullpath)

            clom.git.mv(filepath, newfilename).shell.execute()





from os import path
import os
import shutil
from fnmatch import fnmatch
    
def _get_file_list(root, child, exclude, include):    
    src_dir = root
    if child:
        src_dir = path.join(src_dir, child)
        
    files = []
    for filename in os.listdir(src_dir):
        part = filename
        if child:
            part = path.join(child, filename)
            
        src = path.join(root, part)

        skip = False
        if exclude:
            for p in exclude:
                if fnmatch(filename, p):
                    skip = True
                    break

        if skip:
            continue

        if not path.isdir(src) and include:
            skip = True
            for p in include:
                if fnmatch(filename, p):
                    skip = False
                    break

        if skip:
            continue

        if path.isdir(src):
            files.extend(_get_file_list(root, part, exclude=exclude, include=include))
        else:
            files.append(part)
    
    return files
        
def get_file_list(root, exclude=None, include=None):
    return _get_file_list(root, child=None, exclude=exclude, include=include)
    
def sync_dir_to(src_dir, dst_dir, exclude=None, include=None, ignore_existing=False):
    if not path.exists(dst_dir):
        os.makedirs(dst_dir)

    files_to_copy = get_file_list(src_dir, exclude=exclude, include=include)
    for filename in files_to_copy:
        src = path.join(src_dir, filename)
        dst = path.join(dst_dir, filename)
                
        if ignore_existing and path.exists(dst):
            continue
        else:
            yield (filename, dst)
            shutil.copy2(src, dst)

def sync_lua_dir_to(src_dir, dst_dir, exclude=None, include=None):
    if not path.exists(dst_dir):
        os.makedirs(dst_dir)

    if not include:
        include = []
    if '*.lua' not in include:
        include.append('*.lua')
        
    files_to_copy = get_file_list(src_dir, exclude=exclude, include=include)
    for filename in files_to_copy:
        name, ext = path.splitext(filename)
        if name.endswith('/init'):
            name = name[:-len('/init')]
            
        distname = name.replace('/', '_').replace('.', '_') + ext
        
        src = path.join(src_dir, filename)
        dst = path.join(dst_dir, distname)

        yield (filename, dst)
        
        shutil.copy2(src, dst)